import { PostNodeBuilder, Renderer } from "mobiledoc-kit";
import fs from "fs";
import metadataParser from "markdown-yaml-metadata-parser";
//import { Convertor } from "./Convertor";
//import { StackedImage } from "./ImageProcessor";
import path from "node:path";
import { downloadImagefromURL, uploadToGhost } from "./images";
import axios from "axios";

const urlExist = async (url: string) => {
  if (typeof url !== "string") {
    throw new TypeError(`Expected a string, got ${typeof url}`);
  }

  const response = await axios.head(url).catch(() => {});

  return (
    response !== undefined && (response.status < 400 || response.status >= 500)
  );
};

const renderer: any = Renderer;

if (renderer) {
  console.log("Renderer enabled");
}

export class BlockConvertor {
  builder: any = new PostNodeBuilder();
  filename: string;
  // The markdown bits (preamble and content)
  //
  metadata: any;
  content = "";
  featureImage: string | undefined;

  /**
   * Constructor
   *
   * @param filename the file name
   */
  constructor(filename: string) {
    this.filename = filename;
    this.initialize(filename);
  }

  setFeatureImage(url: string): void {
    this.featureImage = url;
  }

  /**
   * Initialize/reset the convertor class.
   *
   * @param filename The file to be loaded
   */
  public initialize(filename: string): BlockConvertor {
    this.builder = new PostNodeBuilder();
    let content = fs.readFileSync(filename, {
      encoding: "utf8",
      flag: "r",
    });
    const parsed = metadataParser(content);
    this.metadata = parsed.metadata;
    this.content = parsed.content;
    return this;
  }

  /**
   * Process the file
   *
   * @param api the ghost API object
   * @returns Nothing really.
   */
  public async process(api: any) {
    // Check if the entry is already there
    let exists = await urlExist(
      process.env.BASE_URL + "/" + this.metadata.slug
    );
    console.log(`process: ${this.filename} - ${exists}`);
    if (exists) return;

    let buffer = await this.processImages(api, "./temp");
    //console.log(buffer);

    let markdownSection = this.builder.createCardSection("markdown", {
      markdown: buffer,
    });

    let post = this.builder.createPost([markdownSection]);

    let isDraft =
      "draft" in this.metadata ? (this.metadata["draft"] ? true : false) : true;

    let result = JSON.stringify({
      title: this.metadata.title,
      slug: this.metadata.slug,
      mobiledoc: `${JSON.stringify(renderer.render(post, "0.3.1"))}`,
      status: "published",
      visibility: isDraft ? "paid" : "public",
      created_at: this.metadata["date"],
      updated_at: this.metadata["date"],
      published_at: this.metadata["date"],
      tags: this.metadata["tags"],
      author: process.env.AUTHOR_EMAIL,
      feature_image:
        this.featureImage === "@@@undefined"
          ? "/media/generic.jpg"
          : this.featureImage,
      // feature_image_alt: this.metadata["cover"],
      // feature_image_caption: this.metadata["cover"],
    });

    if (process.env.PUSH_POSTS === "true") {
      let retries = 0;
      let pass = true;
      while (retries < 5 && pass) {
        await api.posts
          .add(JSON.parse(result))
          .then(() => (pass = false))
          .catch((r: any) => {
            console.log(JSON.stringify(r));
            retries++;
          });
      }
    } else {
      console.log(
        "Env PUSH_POST is configured so no posts are saved to the ghost instance."
      );
    }

    return result;
  }

  /**
   * Identifies images and tries to process them
   * @param api the ghost api reference
   * @param basepath the path where to save images hosted online
   */
  public async processImages(api: any, basepath: string): Promise<string> {
    // collect images in an array
    //let images: Record<string, StackedImage> = {};
    const imageRegex = /!\[[^\]]*\]\((.*?)\s*("(?:.*[^"])")?\s*\)/g;
    let tempbuffer = this.content;

    for (const item of [
      ...tempbuffer.matchAll(imageRegex),
      ["", "@@@" + this.metadata.cover, ""],
    ]) {
      let url = item[1];
      let imageURL = url;
      let imagePath = url;

      let localpath = url;

      if (url.startsWith("@@@")) {
        localpath = "" + process.env.HEAD_IMAGE_PATH + url.slice(3);
      } else {
        localpath = path.join(path.dirname(this.filename), url);
      }

      // Download images locally if not already
      if (url.toLowerCase().startsWith("http")) {
        // Download the image locally
        localpath = path.join(basepath, path.basename(url));
        await downloadImagefromURL(url, localpath);
      }

      // Upload the image to ghost
      if (imagePath !== "@@@undefined") {
        if (api) {
          let reference = await uploadToGhost(api, localpath);
          imageURL = reference.url.replace("" + process.env.BASE_URL, "");
        }

        // Set the feature image if any
        if (imagePath.startsWith("@@@")) {
          this.setFeatureImage(imageURL);
        } else {
          // replace the image string with the new URL
          console.log(
            "  replace",
            imagePath,
            "with",
            imageURL,
            "for",
            this.filename
          );
          tempbuffer = tempbuffer.replaceAll(imagePath, imageURL);
        }
      }
    }
    return tempbuffer;
  }
}
